@@ -17,11 +17,12 @@ from .injsys import ASPUStandbyHandler, BOPSRampStandbyHandler, \
 from .lillrf import LILLRF, DevLILLRF
 from .machshift import MachShift
 from .modltr import LIModltr
-from .orbit_interlock import BPMInterlock
+from .orbit_interlock import BPMOrbitIntlk, BaseOrbitIntlk, OrbitInterlock
 from .psconv import PSProperty, StrengthConv
 from .pssofb import PSCorrSOFB, PSApplySOFB
 from .pwrsupply import PowerSupply, PowerSupplyPU
-from .rf import RFGen, ASLLRF, BORFCavMonitor, SIRFCavMonitor, RFCav
+from .rf import RFGen, ASLLRF, BORFCavMonitor, SIRFCavMonitor, RFCav, \
+    RFKillBeam
 from .screen import Screen
 from .sofb import SOFB
 from .syncd import DevicesSync
