@@ -19,6 +19,8 @@ class BPM(_Device):
         'GEN_AArrayData', 'GEN_BArrayData', 'GEN_CArrayData', 'GEN_DArrayData',
         'GEN_XArrayData', 'GEN_YArrayData', 'GEN_SUMArrayData',
         'GEN_QArrayData',
+        'GEN_RawXArrayData', 'GEN_RawYArrayData', 'GEN_RawSUMArrayData',
+        'GEN_RawQArrayData',
         'SPPosX-Mon', 'SPPosY-Mon', 'SPSum-Mon', 'SPPosQ-Mon',
         'SPAmplA-Mon', 'SPAmplB-Mon', 'SPAmplC-Mon', 'SPAmplD-Mon',
         'PosX-Mon', 'PosY-Mon', 'Sum-Mon', 'PosQ-Mon',
@@ -70,7 +72,7 @@ class BPM(_Device):
         """."""
         # call base class constructor
         if not _BPMSearch.is_valid_devname(devname):
-            raise ValueError(devname + ' is no a valid BPM or PBPM name.')
+            raise ValueError(devname + ' is not a valid BPM or PBPM name.')
 
         properties = set(BPM._properties)
         if _BPMSearch.is_photon_bpm(devname):
@@ -498,6 +500,26 @@ class BPM(_Device):
         """."""
         self['QPosCal-Sel'] = val
 
+    @property
+    def mtraw_posx(self):
+        """Multi turn raw X array data."""
+        return self['GEN_RawXArrayData']
+
+    @property
+    def mtraw_posy(self):
+        """Multi turn raw Y array data."""
+        return self['GEN_RawYArrayData']
+
+    @property
+    def mtraw_possum(self):
+        """Multi turn raw sum array data."""
+        return self['GEN_RawSUMArrayData']
+
+    @property
+    def mtraw_posq(self):
+        """Multi turn raw Q array data."""
+        return self['GEN_RawQArrayData']
+
     @property
     def sp_posx(self):
         """."""
@@ -779,6 +801,9 @@ class BPM(_Device):
 class FamBPMs(_Devices):
     """."""
 
+    TIMEOUT = 10
+    RFFEATT_MAX = 30
+
     class DEVICES:
         """."""
         SI = 'SI-Fam:DI-BPM'
@@ -810,6 +835,26 @@ class FamBPMs(_Devices):
                 self._mturn_flags[pvo.pvname] = _Flag()
                 pvo.add_callback(self._mturn_set_flag)
 
+    def set_attenuation(self, value=RFFEATT_MAX, timeout=TIMEOUT):
+        """."""
+        for bpm in self:
+            bpm.rffe_att = value
+
+        mstr = ''
+        okall = True
+        t0 = _time.time()
+        for bpm in self:
+            tout = timeout - (_time.time() - t0)
+            if not bpm._wait('RFFEAtt-RB', value, timeout=tout):
+                okall = False
+                mstr += (
+                    f'\n{bpm.devname:<20s}: ' +
+                    f'rb {bpm.rffe_att:.0f} != sp {value:.0f}')
+
+        print('RFFE attenuation set confirmed in all BPMs', end='')
+        print(', except:' + mstr if mstr else '.')
+        return okall
+
     def get_slow_orbit(self):
         """Get slow orbit vectors.
 
@@ -892,8 +937,12 @@ class FamBPMs(_Devices):
         """
         if acq_rate.lower().startswith('monit1'):
             acq_rate = self.csbpm.AcqChan.Monit1
-        else:
+        elif acq_rate.lower().startswith('fofb'):
             acq_rate = self.csbpm.AcqChan.FOFB
+        elif acq_rate.lower().startswith('tbt'):
+            acq_rate = self.csbpm.AcqChan.TbT
+        else:
+            raise ValueError(acq_rate + ' is not a valid acquisition rate.')
 
         if repeat:
             repeat = self.csbpm.AcqRepeat.Repetitive
