@@ -1,7 +1,6 @@
 """E-Gun devices."""
 
 import time as _time
-from datetime import timedelta as _timedelta
 import numpy as _np
 
 from ..pwrsupply.psctrl.pscstatus import PSCStatus as _PSCStatus
@@ -194,6 +193,17 @@ class EGHVPS(_Device):
     def voltage(self, value):
         self['voltoutsoft'] = value
 
+    def cmd_enable(self, timeout=DEF_TIMEOUT):
+        """Enable."""
+        if self['enstatus'] == self.PWRSTATE.Off:
+            self['enable'] = self.PWRSTATE.Off
+            _time.sleep(1)
+
+        self['enable'] = self.PWRSTATE.On
+        if not self._wait('enstatus', self.PWRSTATE.On, timeout=timeout):
+            return False
+        return True
+
     def cmd_turn_on(self, timeout=DEF_TIMEOUT):
         """."""
         self['enable'] = self.PWRSTATE.On
@@ -225,6 +235,15 @@ class EGHVPS(_Device):
             _time.sleep(0.1)
         return False
 
+    def wait_current(self, value, tol=1e-3, timeout=DEF_TIMEOUT):
+        """Wait current setpoint to reach value with tolerance 'tol'."""
+        _t0 = _time.time()
+        while _time.time() - _t0 < timeout:
+            if abs(self['currentoutsoft'] - value) < tol:
+                return True
+            _time.sleep(0.1)
+        return False
+
 
 class EGTriggerPS(_Device):
     """Egun Trigger Power Supply Device."""
@@ -437,7 +456,7 @@ class EGun(_Devices):
     FILACURR_RAMP_NRPTS = 10
     FILACURR_RAMP_DURATION = 7*60  # [s]
 
-    def __init__(self):
+    def __init__(self, print_log=True):
         """Init."""
         self.bias = EGBias()
         self.fila = EGFilament()
@@ -471,6 +490,7 @@ class EGun(_Devices):
         self._filacurr_opval = EGun.FILACURR_OPVALUE
         self._filacurr_tol = EGun.FILACURR_TOLERANCE
         self._last_status = ''
+        self._print_log = print_log
 
         super().__init__('', devices)
 
@@ -503,49 +523,79 @@ class EGun(_Devices):
 
     def cmd_switch_to_single_bunch(self):
         """Switch to single bunch mode."""
+        self._update_last_status('Switching EGun mode to SingleBunch...')
+        _time.sleep(0.1)  # needed for InjCtrl IOC to get logs
+
         if not self.connected:
+            self._update_last_status('ERR:EGun device not connected. Aborted.')
             return False
 
         if not self.trigps.cmd_disable_trigger():
+            self._update_last_status('ERR:Could not disable EGun Trigger.')
             return False
 
         if not self.pulse.cmd_turn_off_multi_bunch():
+            self._update_last_status('ERR:Could not turn off MultiBunch.')
             return False
 
         if not self.bias.set_voltage(self._bias_sb, tol=self._bias_tol):
+            self._update_last_status(
+                'ERR:Could not set EGun Bias voltage to SB operation value.')
             return False
 
         if not self.trigmultipre.cmd_disable():
+            self._update_last_status('ERR:Could not disable MB pre trigger.')
             return False
         if not self.trigmulti.cmd_disable():
+            self._update_last_status('ERR: Could not disable MB trigger.')
             return False
         if not self.trigsingle.cmd_enable():
+            self._update_last_status('ERR: Could not enable SB trigger.')
             return False
 
-        return self.pulse.cmd_turn_on_single_bunch()
+        if not self.pulse.cmd_turn_on_single_bunch():
+            self._update_last_status('ERR: Could not turn on SingleBunch.')
+            return False
+        self._update_last_status('EGun switched to SingleBunch!')
+        return True
 
     def cmd_switch_to_multi_bunch(self):
         """Switch to multi bunch mode."""
+        self._update_last_status('Switching EGun mode to MultiBunch...')
+        _time.sleep(0.1)  # needed for InjCtrl IOC to get logs
+
         if not self.connected:
+            self._update_last_status('ERR:EGun device not connected. Aborted.')
             return False
 
         if not self.trigps.cmd_disable_trigger():
+            self._update_last_status('ERR:Could not disable EGun Trigger.')
             return False
 
         if not self.pulse.cmd_turn_off_single_bunch():
+            self._update_last_status('ERR:Could not turn off SingleBunch.')
             return False
 
         if not self.bias.set_voltage(self._bias_mb, tol=self._bias_tol):
+            self._update_last_status(
+                'ERR:Could not set EGun Bias voltage to MB operation value.')
             return False
 
         if not self.trigsingle.cmd_disable():
+            self._update_last_status('ERR:Could not disable SB trigger.')
             return False
         if not self.trigmultipre.cmd_enable():
+            self._update_last_status('ERR: Could not disable SB pre trigger.')
             return False
         if not self.trigmulti.cmd_enable():
+            self._update_last_status('ERR: Could not enable MB trigger.')
             return False
 
-        return self.pulse.cmd_turn_on_multi_bunch()
+        if not self.pulse.cmd_turn_on_multi_bunch():
+            self._update_last_status('ERR: Could not turn on MultiBunch.')
+            return False
+        self._update_last_status('EGun configured to MultiBunch!')
+        return True
 
     @property
     def is_single_bunch(self):
@@ -604,6 +654,7 @@ class EGun(_Devices):
 
     def set_hv_voltage(self, value=None, duration=None, timeout=DEF_TIMEOUT):
         """Set HVPS voltage."""
+        self._update_last_status('Setpoint received for HVPS voltage...')
         if not self._check_status_ok():
             self._update_last_status('ERR:MPS or LI Status not ok. Aborted.')
             return False
@@ -622,17 +673,35 @@ class EGun(_Devices):
             self.hvps.voltage = value
             return True
 
+        # if needed, enable HVPS
+        if not self.hvps['enstatus']:
+            self._update_last_status('Sending enable command to HVPS...')
+            _time.sleep(0.1)  # needed for InjCtrl IOC to get logs
+            if not self.hvps.cmd_enable(5):
+                self._update_last_status('ERR:Could not enable HVPS.')
+                return False
+            self._update_last_status('HVPS enabled.')
+        _time.sleep(0.1)  # needed for InjCtrl IOC to get logs
+
         # before voltage setpoints, set leakage current to suitable value
         self._update_last_status(
             f'Setting max. leak current to {self._hv_leakcurr:.3f}mA.')
         self.hvps.current = self._hv_leakcurr
+        if not self.hvps.wait_current(self._hv_leakcurr):
+            self._update_last_status(
+                'ERR:Timed out waiting for HVPS leak current.')
+            return False
+        _time.sleep(0.1)  # needed for InjCtrl IOC to get logs
 
         # if value is lower, do only one setpoint
         if value < self.hvps.voltage:
-            self._update_last_status(f'Setting voltage to {value:.3f}kV.')
+            self._update_last_status(f'Setting voltage to {value:.3f}kV...')
             self.hvps.voltage = value
-            return self.hvps.wait_voltage(
-                value, self._hv_tol, timeout=3*timeout)
+            if self.hvps.wait_voltage(value, self._hv_tol, timeout=3*timeout):
+                self._update_last_status('HVPS Ready!')
+                return True
+            self._update_last_status('ERR:Timed out waiting for HVPS voltage.')
+            return False
 
         # else, do a ramp up
         duration = duration if duration is not None else EGun.HV_RAMP_DURATION
@@ -643,9 +712,10 @@ class EGun(_Devices):
         t_inter = duration / (nrpts-1)
 
         self._update_last_status(f'Starting HVPS ramp to {value:.3f}kV.')
+        _time.sleep(0.1)  # needed for InjCtrl IOC to get logs
         self._update_last_status(
-            'This process will take approximatelly'
-            f' {ydata.size*t_inter:.1f}s.')
+            f'This process will take about {ydata.size*t_inter:.1f}s.')
+        _time.sleep(0.1)  # needed for InjCtrl IOC to get logs
         for i, volt in enumerate(ydata[1:]):
             self.hvps.voltage = volt
             self._update_last_status(
@@ -657,7 +727,7 @@ class EGun(_Devices):
                     self._update_last_status(
                         'ERR:MPS or LI Status not ok. Aborted.')
                     return False
-                if self.hvps.voltage - volt < self._hv_tol:
+                if abs(self.hvps.voltage - volt) < self._hv_tol:
                     break
                 _time.sleep(0.1)
             else:
@@ -681,13 +751,14 @@ class EGun(_Devices):
     def is_fila_on(self):
         """Indicate whether filament is on and in operational current."""
         is_on = self.fila.is_on()
-        is_op_sp = abs(self.fila['currentinsoft']-self._filacurr_opval) < 1e-4
-        is_op_rb = abs(self.fila['currentoutsoft']-self._filacurr_opval) < \
+        is_op_sp = abs(self.fila['currentoutsoft']-self._filacurr_opval) < 1e-4
+        is_op_rb = abs(self.fila['currentinsoft']-self._filacurr_opval) < \
             self._filacurr_tol
         return is_on and is_op_sp and is_op_rb
 
     def set_fila_current(self, value=None):
         """Set filament current."""
+        self._update_last_status('Setpoint received for FilaPS current...')
         if not self._check_status_ok():
             self._update_last_status('ERR:MPS or LI Status not ok. Aborted.')
             return False
@@ -708,9 +779,14 @@ class EGun(_Devices):
 
         # elif value is lower, do only one setpoint
         if value < self.fila.current:
-            self._update_last_status(f'Setting current to {value:.3f}A.')
+            self._update_last_status(f'Setting current to {value:.3f}A...')
             self.fila.current = value
-            return self.fila.wait_current(value, self._filacurr_tol)
+            if self.fila.wait_current(value, self._filacurr_tol):
+                self._update_last_status('FilaPS Ready!')
+                return True
+            self._update_last_status(
+                'ERR:Timed out waiting for FilaPS current.')
+            return False
 
         # else, do a ramp up
         duration = EGun.FILACURR_RAMP_DURATION
@@ -722,6 +798,7 @@ class EGun(_Devices):
 
         self._update_last_status(
             f'Starting filament ramp to {value:.3f} A.')
+        _time.sleep(0.1)  # needed for InjCtrl IOC to get logs
         for i, cur in enumerate(ydata[1:]):
             self.fila.current = cur
             dur = total_steps_duration - i*t_inter
@@ -735,6 +812,7 @@ class EGun(_Devices):
                     'ERR:MPS or LI Status not ok. Aborted.')
                 return False
         self._update_last_status('FilaPS Ready!')
+        return True
 
     def _get_ramp(self, curr_val, goal, nrpts, max_val, power=2):
         xdata = _np.linspace(0, 1, nrpts)
@@ -761,4 +839,5 @@ class EGun(_Devices):
 
     def _update_last_status(self, status):
         self._last_status = status
-        print(status)
+        if self._print_log:
+            print(status)
